
;**********************************************************************
;                                                                     *
;    Filename:	    lcd.asm   4 bit lcd routine                       *
;    Date:          December 2010                                     *
;    File Version:  V1                                                *
;                                                                     *
;                                                                     *
;   Using Pins 1-7 of Port B   based on 4 Mhz Crystal				  *


		list      p=pic16F877A           ; list directive to define processor
		#include <p16F877A.inc>        ; processor specific variable definitions
	
		__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC




;***** VARIABLE DEFINITIONS

		cblock	0x020				; Variable memory start address

	
		t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16  ; lcd top row
		b1,b2,b3,b4,b5,b6,b7,b8,b9,b10,b11,b12,b13,b14,b15,b16	; lcd bottom row
		t0,b0    					; lcd control

		d1,d2,d3					; delay work files

		lcd_flags,lcd_temp			; lcd work files
		CENTIG

		 BIN, huns, tens, ones, count		; ascii3 converter

		endc						; denotes end of ram memory 


		ORG     0x000 

bootup	clrf 	INTCON
		clrf	PCLATH
		goto	start


		ORG		 0x004 					; Interupt Service Routine 	
		retfie						; return from isr

							


		

start	clrf	PORTA				; Set up I/O Ports
		clrf	PORTB
		banksel	TRISA
		movlw	0xFF				; make PortA Analogue inputs
		movwf	TRISA
		clrf	TRISB				; make PortB Digital outputs
		banksel 0
	

		call 	lcd_init			; Initialise the LCD module





mainloop
		call	topdata				; load the lcd top row data message
		call	topsend				; send the top row to the lcd
		call	delay1sec
		call	delay1sec

		call	cleardata			; clear the screen
		call	topsend				; output the new data 
		call	delay1sec			; delay to read message

		call	adcdata				; dispay adc message
		call	topsend				; output the new data 
		call	delay1sec
		call	delay1sec

;		your adc routine
		movlw 	b'11100110'           ; assume this is the ADRESL returned value
		movwf	BIN
		call	ascii3		 	    ; convert ADRESL to 3 digit ascii value for display
		
		movf	huns,W		        ; load ascii values to display
		movwf	t12
		movf	tens,W		        ; load ascii values to display
		movwf	t13
		movf	ones,W		        ; load ascii values to display
		movwf	t14
	

		call	topsend				; output the new data 

		call	delay1sec			; delay to read message


loop		goto	loop			; program now just loops forever




; ***************************************************************************
;
;  All the code below are subroutines 'called'  by the main program above





;****************************************************************************
; Routines to display on a 2x16 character LCD display.
;
; LcdInit - call this subroutine before using the display. It uses
;           bits 7-1 of PORTB, leaving bit-0 free (although it will
;           be configured as an output).
;
; Lcd_C -   Sends the byte in W as a command to the display. Checks
;           if busy first. 
;
; Lcd_D -   Sends the byte in W as data. Checks for busy first.
;
; Address of first line on the LCD is 0
; Address of second line is 64
;
; by Ron Kreymborg AU
;******************************************************************



lcd_com	macro	arg1
		movlw	arg1
		call	lcd_c
		endm
	

; Flags -
lcd_Bflg	equ	0
lcd_RSflg	equ	1

; I/O portB - 
lcd_BUSY	equ	7					; input - LCD is busy (0x80)
lcd_R_W		equ	3	;RB3			; output - LCD Read/Write (0x08)
lcd_RS		equ	2	;RB2			; output - Register Select (0x04)
lcd_E		equ	1	;RB1			; output - LCD Enable (0x02)

					;RB7 - RB4 datalines to lcd pins 14 - 11


lcd_init
		banksel PORTB
		clrf	PORTB
		banksel	TRISB
		clrf 	TRISB				; all outputs and low to start
		bcf	lcd_flags,lcd_RSflg
		call	delay15				; 15mSec power up delay		
		movlw	b'00110000'			; 8-bit mode
		banksel PORTB
		movwf	PORTB
		call	lcd_clk
		call	delay5				; 4mSec wait
		movlw	b'00110000'			; 8-bit mode
		movwf	PORTB
		call	lcd_clk
		call	delay1				; 1 mSec wait
		movlw	b'00110000'			; 8-bit mode
		movwf	PORTB
		call	lcd_clk
		call	delay5					; 4mSec wait
		movlw	b'00100000'			; set for 4-bit
		movwf	PORTB
		call	lcd_clk

; Reset sequence is done - initialise for us

		lcd_com	b'00101000'			; 4-bits, 2-lines, 5x7
		lcd_com	b'00001000'			; display off, cursor off, blink off
		lcd_com	b'00001100'			; display on
		lcd_com	b'00000001'			; clear display
		lcd_com	b'00000110'			; increment, no display shift
		clrf	PCLATH
		return

	
	
; Check whether the LCD is busy. Loop until it isn't. When
; it's free, output the byte passed in W, MS nibble first.

lcd_d	bsf		lcd_flags,lcd_RSflg	; set RS flag for data
		goto	lcd_o1
lcd_c	bcf		lcd_flags,lcd_RSflg	; clear RS flag for commands
lcd_o1	movwf	lcd_temp			; save control word
		movlw	b'11110000'			; RB7-4 as inputs for busy
		banksel	TRISB
		movwf	TRISB
		movlw	1 << lcd_R_W
		banksel	PORTB
		movwf	PORTB			; set up for read
		
lcd_o2	bcf		lcd_flags,lcd_Bflg	; assume not busy
		bsf		PORTB,lcd_E			; clock high
		nop							; little wait
		btfsc	PORTB,lcd_BUSY		; busy set?
		bsf		lcd_flags,lcd_Bflg	; yes
		bcf		PORTB,lcd_E			; clock low
		nop							; little wait
		nop
		call	lcd_clk				; get low bits but ignore (for now)
		btfsc	lcd_flags,lcd_Bflg	; was it busy?
		goto	lcd_o2				; yes
		banksel	TRISB				; RB7-4 outputs again
		clrf	TRISB
		banksel PORTB
		movf	lcd_temp,w			; get word to send
		call	lcd_o3				; send high nibble
		swapf	lcd_temp,w			; get word again
lcd_o3	andlw	0xf0				; just high bits (clears all control bits)
		movwf	PORTB
		btfsc	lcd_flags,lcd_RSflg	; was lcd_RS set?
		bsf		PORTB,lcd_RS		; yes
		call	lcd_clk				; write current nibble
		clrf	PCLATH
		return
		
	
lcd_clk	bsf		PORTB,lcd_E			; pulse E line high
		nop
		bcf		PORTB,lcd_E
		clrf	PCLATH
		return
	



; ***************************************************************************
; 
; 	LCD Data Output Routines
; 	========================


topsend							; output top line of lcd data
	    movf	t0,w			
		iorlw	0x80			; add address for command line	
		call    lcd_c			
		movf	t1,w
		call	lcd_d
		movf	t2,w
		call	lcd_d
		movf	t3,w
		call	lcd_d
		movf	t4,w
		call	lcd_d
		movf	t5,w
		call	lcd_d
		movf	t6,w
		call	lcd_d
		movf	t7,w
		call	lcd_d
		movf	t8,w
		call	lcd_d
		movf	t9,w
		call	lcd_d
		movf	t10,w
		call	lcd_d
		movf	t11,w
		call	lcd_d
		movf	t12,w
		call	lcd_d
		movf	t13,w
		call	lcd_d
		movf	t14,w
		call	lcd_d
		movf	t15,w
		call	lcd_d
		movf	t16,w
	
		clrf	PCLATH
		return	



	
	
; *****************************************************************************
;
;   LCD Display Messages
;	====================


topdata movlw	0x00				; lcd top row sata message 1			
		movwf	t0				
		movlw	'H'		
		movwf	t1				
		movlw	'E'
		movwf	t2
		movlw	'L'
		movwf	t3	
		movlw	'L'
		movwf	t4
		movlw	'O'
		movwf	t5
		movlw	' '
		movwf 	t6
		movlw	' '
		movwf	t7
		movlw 	' '
		movwf	t8
		movlw	' '
		movwf	t9
		movlw	' '
		movwf	t10
		movlw	' '
		movwf	t11
		movlw	' '
		movwf	t12
		movlw	' '
		movwf	t13
		movlw	' '
		movwf	t14
		movlw	' '
		movwf	t15
		movlw	' '
		movwf	t16

		return
	



cleardata movlw	0x00				; clear screen
		movwf	b0				
		movlw	' '			
		movwf	b1				
		movwf	b2
		movwf	b3	
		movwf	b4
		movwf	b5
		movwf 	b6
		movwf	b7
		movwf	b8
		movwf	b9
		movwf	b10
		movwf	b11
		movwf	b12
		movwf	b13
		movwf	b14
		movwf	b15
		movwf	b16

		return


adcdata movlw	0x00				; lcd top row sata message 1			
		movwf	t0				
		movlw	'A'		
		movwf	t1				
		movlw	'D'
		movwf	t2
		movlw	'C'
		movwf	t3	
		movlw	' '
		movwf	t4
		movlw	'R'
		movwf	t5
		movlw	'e'
		movwf 	t6
		movlw	's'
		movwf	t7
		movlw 	'u'
		movwf	t8
		movlw	'l'
		movwf	t9
		movlw	't'
		movwf	t10
		movlw	' '
		movwf	t11
		movlw	' '
		movwf	t12
		movlw	' '
		movwf	t13
		movlw	' '
		movwf	t14
		movlw	' '
		movwf	t15
		movlw	' '
		movwf	t16

		return
	





; ***************************************************************************
;
;	Delay routines based on a 4 meg Crystal
;	========================================



delay1		movlw	0xC7				; delay 1ms  - 998 cycles
			movwf	d1
			movlw	0x01
			movwf	d2
dly1_1
			decfsz	d1, f
			goto	dly1_2
			decfsz	d2, f
dly1_2		goto	dly1_1

			return


		
delay5		movlw	0xE7			;delay 5ms  - 4998 cycles
			movwf	d1
			movlw	0x04
			movwf	d2
dly5_1
			decfsz	d1, f
			goto	dly5_2
			decfsz	d2, f
dly5_2		goto	dly5_1
			retlw	0x00

delay15		call	delay5	
			call	delay5
			call	delay5

			return



delay250	movlw	d'250'			; delay 250 ms 
			movwf	d1
dly250_1	movlw	0xC7
			movwf	d2
			movlw	0x01
			movwf	d3
dly250_2
			decfsz	d2, f
			goto	dly250_3
			decfsz	d3, f
dly250_3	goto	dly250_2
			decfsz	d1	,f
			goto	dly250_1

			return



delay500	call	delay250		; delay 500ms
			call	delay250

			return


delay1sec	call	delay500		; delay 1sec
			call	delay500

			return


delay5sec	call 	delay1sec		; delay 5sec
			call 	delay1sec
			call	delay1sec
			call	delay1sec
			call	delay1sec

			return

			
		
; -----------------------------------------------------------------------------
;
;	ASCII 3 DIGIT CONVERT 
;	=====================


;in: BIN
;out huns. tens, ones

;uses ADD-3 algoerthm

ascii3
	movlw 8
	movwf count
	clrf huns
	clrf tens
	clrf ones
BCDADD3
	movlw 5
	subwf huns, 0
	btfsc STATUS, C
	CALL ADD3HUNS
	movlw 5
	subwf tens, 0
	btfsc STATUS, C
	CALL ADD3TENS
	movlw 5
	subwf ones, 0
	btfsc STATUS, C
	CALL ADD3ONES
	decf count, 1
	bcf STATUS, C
	rlf BIN, 1
	rlf ones, 1
	btfsc ones,4 ; 
	CALL CARRYONES
	rlf tens, 1
	btfsc tens,4 ; 
	CALL CARRYTENS
	rlf huns,1
	bcf STATUS, C
	movf count, 0
	btfss STATUS, Z
	GOTO BCDADD3
	movf huns, 0 ; add ASCII Offset
	addlw h'30'
	movwf huns
	movf tens, 0 ; add ASCII Offset
	addlw h'30'
	movwf tens
	movf ones, 0 ; add ASCII Offset
	addlw h'30'
	movwf ones
	RETURN
ADD3HUNS
	movlw 3
	addwf huns,1
	RETURN
ADD3TENS
	movlw 3
	addwf tens,1
	RETURN
ADD3ONES
	movlw 3
    addwf ones,1
	RETURN
CARRYONES
	bcf ones, 4
	bsf STATUS, C
	RETURN
CARRYTENS
	bcf tens, 4
	bsf STATUS, C
	RETURN


			END

	
